# default calibration data for lenses
# TW60
# TW90
#### TBD: add TW50, TW80, TW37

import logging as log
import json

def loadDefaultData(lens:str='NA'):
    data = {}
    
    def addTW60ApertureData(data):
        # ...\Designs\TL1250\Gauge studies\FNum test\FNum measurements 230403.xlsx
        data['AP'] = {
            "type": "des",
            "idx": 6,
            "count": 5,
            "xAxis": "iris step",
            "xMin": 0,
            "xMax": 75,
            "yAxis": "Aperture",
            "cp1Type": "FL [mm]",
            "cp2Type": "",
            "cp1": [48.5, 45.1, 37.0, 26.3, 12.1],
            "cp2": [0, 0, 0, 0, 0],
            "coef": [
                [2.381870e-01, -5.983423e-04, -7.805547e-05, 5.157000e-07],
                [2.491681e-01, -1.248998e-03, -6.371687e-05, 4.072398e-07],
                [2.632689e-01, -2.139779e-03, -4.047746e-05, 2.063623e-07],
                [2.660441e-01, -2.262262e-03, -3.756097e-05, 1.823667e-07],
                [2.710063e-01, -2.212058e-03, -4.181084e-05, 2.265186e-07],
            ],
        }
        # ...\Designs\TL1250\Gauge studies\FNum test\FNum measurements 230403.xlsx
        data['iris'] = {
            "type": "des",
            "idx": 15,
            "count": 3,
            "xAxis": "iris step",
            "xMin": 0,
            "xMax": 75,
            "yAxis": "Dia (short)",
            "cp1Type": "FL [mm]",
            "cp2Type": "",
            "cp1": [45.1, 26.3, 12.1],
            "cp2": [0, 0, 0],
            "coef": [
                [2.389286e01, -3.982144e-01],
                [1.403517e01, -2.339195e-01],
                [6.581851e00, -1.096975e-01],
            ],
        }
        return data

    ########## main data load functions ########
    ########### TW50 default data ##############
    if 'TW50' in lens:
        data = {
            "manufName": "Theia Technologies",
            "fam": "TW50",
            "pn": "TL410P R6",
            "flMin": 4.15,
            "flMax": 9.5,
            "fnum": 1.4,
            "zoomSteps": 4073,
            "zoomPI": 154,
            "focusSteps": 9353,
            "focusPI": 8652,
            "irisSteps": 75,
            "ffl": 12.5,
            "ihMax": 9.4,
            "odMin": 0.5
            }
        # data['dist'] = 
        # data['tracking'] =
        # data['FL'] =
        # data = addTW50ApertureData(data)
        log.info("TW50 default data stored")
        ########### end of TW50 default data #######

    ########### TW80 default data ##############
    elif 'TW80' in lens:
        data = {
            "manufName": "Theia Technologies",
            "fam": "TW80",
            "pn": "TL410P N6",
            "flMin": 4.15,
            "flMax": 9.5,
            "fnum": 1.4,
            "zoomSteps": 4017,
            "zoomPI": 136,
            "focusSteps": 9269,
            "focusPI": 8674,
            "irisSteps": 75,
            "ffl": 12.5,
            "ihMax": 9.4,
            "odMin": 0.5
            }
        # data['dist'] = 
        # data['tracking'] =
        # data['FL'] =
        # data = addTW50ApertureData(data)
        log.info("TW80 default data stored")
        ########### end of TW80 default data #######
        
    ########### TW60 default data ##############
    elif 'TW60' in lens:
        data = {
            "manufName": "Theia Technologies",
            "fam": "TW60",
            "pn": "TL1250P R6",
            "flMin": 12.36,
            "flMax": 48.5,
            "fnum": 1.8,
            "zoomSteps": 3256,
            "zoomPI": 3147,
            "focusSteps": 8466,
            "focusPI": 8031,
            "irisSteps": 75,
            "ffl": 12.5,
            "ihMax": 9.4,
            "odMin": 1.5
            }
        data['dist'] = {
            "type": "cal",
            "idx": 2,
            "count": 9,
            "xAxis": "image ht [mm]",
            "yAxis": "ang [deg]",
            "cp1Type": "focal length",
            "cp2Type": "1000/obj dist",
            "cp1": [
                12.57234,
                13.97656,
                16.34685,
                20.89305,
                25.82942,
                30.70073,
                37.94332,
                41.69358,
                44.76839,
            ],
            "cp2": [0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01],
            "coef": [
                [
                    -0.31233819620290876,
                    -4.841353983458336,
                    -0.025409849595849306,
                    0.018987897779012004,
                    0.0025148803040172646,
                    -0.0010212288904498912,
                    -8.015016310313382e-05,
                ],
                [
                    -0.0026201436491238173,
                    -4.06961437325067,
                    0.002581778106112695,
                    -0.005391675398640108,
                    -3.3899059186468066e-05,
                    -0.0001324287989808055,
                    2.577058283848103e-06,
                ],
                [
                    -0.0007567140608406459,
                    -3.4570380509063714,
                    0.0032194702495361004,
                    -0.003546753736917385,
                    -0.00020115046244570587,
                    -6.692721477250293e-05,
                    6.192546220761145e-06,
                ],
                [
                    -0.0011692517663053005,
                    -2.6879243168872082,
                    -0.0004474307814071429,
                    -0.0016290332700571201,
                    5.9730434417427926e-05,
                    -1.3749316011504492e-05,
                    4.572432475816435e-07,
                ],
                [
                    -0.0021383878467612677,
                    -2.1712968104312913,
                    -0.0007454731042883671,
                    7.832455965604178e-05,
                    8.121154708431149e-05,
                    -1.9606422481530556e-05,
                    -2.0991801249364577e-06,
                ],
                [
                    -0.004326796590160402,
                    -1.8205360126540853,
                    0.0004332811032398695,
                    0.0006788564625753822,
                    -1.784335613104938e-05,
                    -1.2954983968754755e-05,
                    3.3213108773577106e-07,
                ],
                [
                    -0.003138204191038394,
                    -1.4666577359022064,
                    -0.0007064864270575007,
                    0.0012802683333857665,
                    0.00010205367151946463,
                    -2.1360259402051036e-05,
                    -2.1631022073186387e-06,
                ],
                [
                    -0.002399131702081612,
                    -1.33782759626553,
                    -0.00203901523022636,
                    0.0014674713301577416,
                    0.0002762686596367444,
                    -2.2276861530550996e-05,
                    -8.53448906075942e-06,
                ],
                [
                    -0.002522106209222052,
                    -1.2371014906381497,
                    -0.001377508309309832,
                    0.0016505029509675771,
                    0.00014837739403833068,
                    -1.5913271028491694e-05,
                    -4.063594988640064e-06,
                ],
            ],
            "coefInv": [
                [
                    -0.06858440413126714,
                    -0.20695838601993966,
                    -9.98198666469729e-05,
                    -3.0340002630891725e-05,
                    1.675280384663493e-07,
                    7.063467537042979e-08,
                ],
                [
                    -0.0005107889388636627,
                    -0.24583651695322412,
                    3.147820300333646e-05,
                    2.2066493955734082e-05,
                    1.0540115370803073e-08,
                    1.193311588988384e-08,
                ],
                [
                    0.0004004577711149523,
                    -0.28933061019605083,
                    3.53765354689729e-05,
                    2.6744635650266306e-05,
                    -3.5116012725043847e-09,
                    1.9395611558679608e-08,
                ],
                [
                    -0.00042968915542269503,
                    -0.37204565922996974,
                    -2.3944590366035727e-05,
                    3.17998728988103e-05,
                    4.611664813343537e-07,
                    2.1542381797146235e-08,
                ],
                [
                    -0.0012600672996816113,
                    -0.4605613796603709,
                    -2.076120387724974e-05,
                    -3.02678799565186e-06,
                    2.8056361246831223e-07,
                    1.801689207459176e-07,
                ],
                [
                    -0.002335277028152583,
                    -0.5492826770977438,
                    5.9435033761910655e-05,
                    -6.258737834082516e-05,
                    -3.8583010888259666e-07,
                    3.617807573507341e-07,
                ],
                [
                    -0.0024757437947695708,
                    -0.6817915230390219,
                    -7.937417459160258e-05,
                    -0.00028352038914744365,
                    5.839242087046858e-06,
                    2.183886443865156e-06,
                ],
                [
                    -0.0034265467266372555,
                    -0.7474281559931831,
                    -1.3569748663526606e-05,
                    -0.0004722025510407848,
                    2.6181165031811645e-06,
                    3.9384021719138035e-06,
                ],
                [
                    -0.0030313056804933142,
                    -0.8082671447206948,
                    -0.0001712418598558725,
                    -0.0007252796282529922,
                    5.216845556869472e-06,
                    3.9591476274599465e-06,
                ],
            ],
        }
        data['tracking'] = {
            "type": "cal",
            "idx": 1,
            "count": 4,
            "xAxis": "zoom step",
            "yAxis": "focus step",
            "cp1Type": "1000/obj dist",
            "cp2Type": "none",
            "cp1": [0.01, 100.0, 200.0, 400.0],
            "cp2": [0, 0, 0, 0],
            "coef": [
                [
                    6563.84094760557,
                    -13.2161832565621,
                    0.0118923822553627,
                    -6.72626619762397E-06,
                    2.63805820891675E-09,
                    -5.95587901410704E-13,
                    5.58402845525731E-17
                ],
                [
                    6228.5009715106,
                    -12.8594062628038,
                    0.0117348933158789,
                    -6.70292345580927E-06,
                    2.63184404939814E-09,
                    -5.9081308489072E-13,
                    5.49437664681417E-17
                ],
                [
                    5790.7921187882,
                    -11.8835319357794,
                    0.00987287451319531,
                    -4.56756772661166E-06,
                    1.40310039319681E-09,
                    -2.57994433456135E-13,
                    2.09812920425618E-17
                ],
                [
                    4837.81541797278,
                    -10.7415597860671,
                    0.00908753889231266,
                    -3.92030447094245E-06,
                    9.57651656399763E-10,
                    -1.09334452088126E-13,
                    3.14128090316752E-18
                ],
            ],
        }
        data['FL'] = {
            "type": "cal",
            "idx": 4,
            "count": 1,
            "xAxis": "focal length",
            "yAxis": "zoom step",
            "cp1Type": "1000/obj dist",
            "cp2Type": "none",
            "cp1": [0.01],
            "cp2": [0],
            "coefInv": [
                [
                    45.7893477357648,
                    -0.0256409513060103,
                    1.66484650420773E-05,
                    -1.42604412893191E-08,
                    7.93467404970622E-12,
                    -2.18236551452107E-15,
                    2.28544716032465E-19
                ]
            ],
            "coef": [
                [
                    6151.08829980658,
                    -254.094159979907,
                    -4.29887501493942,
                    0.558802357411088,
                    -0.0156987411609015,
                    0.000178002596132931,
                    -6.47080786865557E-07
                ]
            ],
        }
        data = addTW60ApertureData(data)
        log.info("TW60 default data stored")
        ########### end of TW60 default data #######

    ########### TW90 default data ##############
    elif "TW90" in lens:
        data = {
            "manufName": "Theia Technologies",
            "fam": "TW90",
            "pn": "TL1250P N6",
            "flMin": 12.36,
            "flMax": 48.5,
            "fnum": 1.8,
            "zoomSteps": 3227,
            "zoomPI": 3119,
            "focusSteps": 8390,
            "focusPI": 7959,
            "irisSteps": 75,
            "ffl": 12.5,
            "ihMax": 9.4,
            "odMin": 1.5
            }
        # ...Designs\Calibrated lenses\Data\1250\Calibrated lenses Becom\Average Distortion.xlsx
        data['dist'] = {
            "type": "cal",
            "idx": 2,
            "count": 9,
            "xAxis": "image ht [mm]",
            "yAxis": "ang [deg]",
            "cp1Type": "focal length",
            "cp2Type": "1000/obj dist",
            "cp1": [12,15,19,24,28,32,36,40,45,50],
            "cp2": [0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01],
            "coef": [
                [0.000411895116796559,-4.74314300755856,-0.00904412309554661,0.00553218246129731,-0.00722740950517411,0.00150469670302169,-0.000148277862881819],
                [0.000324999168975726,-3.74250014852848,-0.0071361188086432,0.00436507894660042,-0.00570267038194852,0.00118725655658425,-0.000116996245521952],
                [0.000250104101339535,-2.8800523993949,-0.00549162198570183,0.00335915981155055,-0.00438850737866442,0.000913656902813659,-9.00348174382642E-05],
                [0.000192834397218223,-2.22056801715285,-0.00423413134646261,0.00258996775322539,-0.00338361174614406,0.000704444577975696,-6.94183328316123E-05],
                [0.000161459820060563,-1.85927675587722,-0.00354522893827492,0.00216857434893156,-0.00283309073260956,0.000589829908166768,-5.81238186214906E-05],
                [0.000137397942046586,-1.58219425648278,-0.00301689398651068,0.00184539814677916,-0.00241088362507868,0.000501929306680228,-4.94617983563437E-05],
                [0.000119723480238114,-1.37866550239003,-0.00262880973466584,0.00160801162860857,-0.00210075474016876,0.000437362616457128,-4.30991800157725E-05],
                [0.000108862355565298,-1.25359514966054,-0.0023903282754535,0.00146213535823155,-0.00191017759444558,0.000397685771989434,-3.91892906063376E-05],
                [0.000103531938827724,-1.19221319137944,-0.00227328647729091,0.00139054228323725,-0.00181664624866443,0.000378213192287811,-3.72703972525122E-05],
                [9.77156343356074E-05,-1.12523603419586,-0.00214557587419709,0.00131242322724574,-0.00171458916507106,0.000356965612903562,-3.51765894730178E-05]
            ],
            "coefInv": [
                [0.000115418552038177,-0.208511919081332,5.62471988614302E-05,1.18646450182578E-05,-1.87052467153934E-08,-2.10908297650415E-08],
                [0.000115418552038942,-0.268379548224083,0.000100405314150968,4.62842103812749E-05,1.03134025257712E-07,4.0004761875733E-08],
                [0.000115418552037685,-0.344949347881075,0.000162154741279473,8.72060948781326E-05,1.97144922850118E-07,7.5094527367587E-08],
                [0.000115418552036829,-0.449952305297254,0.000276364259167126,0.000193572813135045,5.62953883797774E-07,2.64507184599256E-07],
                [0.000115418552037684,-0.541534290119785,0.000402190651488216,0.000343973484906492,1.2375691508727E-06,7.21654216496351E-07],
                [0.00011541855203912,-0.633026422870308,0.000548640503252238,0.000548062942912149,2.30864810345931E-06,1.5822381690781E-06],
                [0.000115418552039866,-0.716344396729625,0.000700678004995873,0.000788802421181216,3.74454001897875E-06,2.90048237142977E-06],
                [0.000115418552039065,-0.786094153781497,0.000844783240772949,0.00104477146311827,5.4471983138769E-06,4.63531252820527E-06],
                [0.000115418552036743,-0.857407748569797,0.00100690508202555,0.001359154974223,7.71463476193542E-06,7.12129883865709E-06],
                [0.000115418552038709,-0.938778734507755,0.00117320672865041,0.00165908120184036,9.77204797859692E-06,9.3097101570855E-06]
            ],
        }
        # ...Designs\Calibrated lenses\Data\Tracking curve calibration testing\Focus step shift from design 230310.xlsx
        data['tracking'] = {
            "type": "cal",
            "idx": 1,
            "count": 4,
            "xAxis": "zoom step",
            "yAxis": "focus step",
            "cp1Type": "1000/obj dist",
            "cp2Type": "none",
            "cp1": [0.01, 100.0, 200.0, 400.0],
            "cp2": [0, 0, 0, 0],
            "coef": [
                [
                    6563.84094760557,
                    -13.2161832565621,
                    0.0118923822553627,
                    -6.72626619762397E-06,
                    2.63805820891675E-09,
                    -5.95587901410704E-13,
                    5.58402845525731E-17
                ],
                [
                    6228.5009715106,
                    -12.8594062628038,
                    0.0117348933158789,
                    -6.70292345580927E-06,
                    2.63184404939814E-09,
                    -5.9081308489072E-13,
                    5.49437664681417E-17
                ],
                [
                    5790.7921187882,
                    -11.8835319357794,
                    0.00987287451319531,
                    -4.56756772661166E-06,
                    1.40310039319681E-09,
                    -2.57994433456135E-13,
                    2.09812920425618E-17
                ],
                [
                    4837.81541797278,
                    -10.7415597860671,
                    0.00908753889231266,
                    -3.92030447094245E-06,
                    9.57651656399763E-10,
                    -1.09334452088126E-13,
                    3.14128090316752E-18
                ],
            ],
        }
        # ...Designs\Calibrated lenses\Data\Tracking curve calibration testing\Focus step shift from design 230310.xlsx
        data['FL'] = {
            "type": "cal",
            "idx": 4,
            "count": 1,
            "xAxis": "focal length",
            "yAxis": "zoom step",
            "cp1Type": "1000/obj dist",
            "cp2Type": "none",
            "cp1": [0.01],
            "cp2": [0],
            "coefInv": [
                [
                    45.7893477357648,
                    -0.0256409513060103,
                    1.66484650420773E-05,
                    -1.42604412893191E-08,
                    7.93467404970622E-12,
                    -2.18236551452107E-15,
                    2.28544716032465E-19
                ]
            ],
            "coef": [
                [
                    6151.08829980658,
                    -254.094159979907,
                    -4.29887501493942,
                    0.558802357411088,
                    -0.0156987411609015,
                    0.000178002596132931,
                    -6.47080786865557E-07
                ]
            ],
        }
        data = addTW60ApertureData(data)
        log.info("TW90 default data stored")
        ########### end of TW60 default data #######

    elif lens == 'NA':
        # no lens family parameter entered
        log.warning('No default data loaded')
    else:
        # lens family parameter unknown
        log.error(f"No default data for {lens}")
    return data


# validate a calibration data file
# read the file contents and validate that the manufacturer key is "Theia".
# Return the file contents.
class ValidateCalibrationFile:
    MANUF_NAME_KEY = "manufName"  # the key in the JSON file with manufacturer's name
    MANUF_NAME = "Theia Technologies"  # manufacturer name for JSON file validation
    LENS_FAM_KEY = "fam"

    def __init__(self, fileName):
        self.fileName = fileName

    def readFile(self):
        with open(self.fileName, "r") as f:
            data = json.load(f)

        if ValidateCalibrationFile.MANUF_NAME_KEY not in data:
            raise ValueError(f"Missing key {ValidateCalibrationFile.MANUF_NAME_KEY} in JSON file.")

        if (data[ValidateCalibrationFile.MANUF_NAME_KEY] != ValidateCalibrationFile.MANUF_NAME):
            raise ValueError(f"Invalid manufacturer name: {data[ValidateCalibrationFile.MANUF_NAME_KEY]}")

        return data
